import io
import sys
import errno

import pretend
import pytest

from configstore.backends.docker_secret import DockerSecretBackend


if sys.version_info[0] == 3:
    builtins_open = 'builtins.open'
else:  # pragma: no cover
    builtins_open = '__builtin__.open'


def test_docker_secret_success(monkeypatch):
    fake_open = pretend.call_recorder(lambda path: io.StringIO(u'sup3r s3cr3t'))
    # StringIO implements __enter__, so our one-liner works for "with open(path)"
    monkeypatch.setattr(builtins_open, fake_open)

    b = DockerSecretBackend()
    value = b.get_setting('APP_SECRET_KEY')

    assert value == 'sup3r s3cr3t'
    assert fake_open.calls == [pretend.call('/run/secrets/APP_SECRET_KEY')]


def test_docker_secret_missing(monkeypatch):
    b = DockerSecretBackend('/does/not/exist/at/all')
    value = b.get_setting('APP_SECRET_KEY')

    assert value is None


def test_docker_secret_bad_config(monkeypatch):
    exc = OSError(errno.EISDIR, 'is a dir')
    fake_open = pretend.call_recorder(pretend.raiser(exc))
    monkeypatch.setattr(builtins_open, fake_open)

    b = DockerSecretBackend()
    with pytest.raises(OSError):
        b.get_setting('APP_SECRET_KEY')


def test_docker_secret_custom_path_success(monkeypatch):
    fake_open = pretend.call_recorder(lambda path: io.StringIO(u'wow s3cr3t'))
    monkeypatch.setattr(builtins_open, fake_open)

    b = DockerSecretBackend('/custom/path')
    value = b.get_setting('APP_SECRET_KEY')

    assert value == 'wow s3cr3t'
    assert fake_open.calls == [pretend.call('/custom/path/APP_SECRET_KEY')]



from configstore.backends.env_var import EnvVarBackend


def test_env_var_success(monkeypatch):
    monkeypatch.setenv('APP_ENVIRONMENT', 'STAGING')

    b = EnvVarBackend()
    value = b.get_setting('APP_ENVIRONMENT')

    assert value == 'STAGING'


def test_env_var_missing(monkeypatch):
    monkeypatch.delenv('APP_DEBUG', raising=False)

    b = EnvVarBackend()
    value = b.get_setting('APP_DEBUG')

    assert value is None



import pytest

from configstore import Store, SettingNotFoundException


class DictBackend:
    def __init__(self, **settings):
        self._settings = settings

    def get_setting(self, key):
        return self._settings.get(key)


def test_store_init():
    Store([])
    Store([DictBackend()])
    # This error is not caught at the moment
    Store([DictBackend])

    with pytest.raises(Exception):
        Store()
    with pytest.raises(Exception):
        Store(None)


def test_store_get_setting():
    store = Store([DictBackend(key='secrets!')])

    value = store.get_setting('key')

    assert value == 'secrets!'


def test_store_get_setting_with_default():
    store = Store([DictBackend(key='secrets!')])

    value = store.get_setting('key', 'default')

    assert value == 'secrets!'


def test_store_get_setting_missing():
    store = Store([])

    with pytest.raises(SettingNotFoundException):
        store.get_setting('key')


def test_store_get_setting_missing_with_default():
    store = Store([DictBackend()])

    value = store.get_setting('key', 'default value')

    assert value == 'default value'


def test_store_interpolate():
    store = Store([DictBackend(environment='staging')])

    s = store.interpolate('before ${environment} after')

    assert s == 'before staging after'


def test_store_interpolate_none_default():
    store = Store([])

    assert store.get_setting('foo', None) is None


def test_store_get_setting_interpolate_value():
    store = Store([DictBackend(
        environment='staging',
        secret_key='42-${environment}-secrets!',
    )])

    value = store.get_setting('secret_key')

    assert value == '42-staging-secrets!'


def test_store_get_setting_interpolate_default():
    store = Store([DictBackend(service_host='cool-db-server:6000')])

    value = store.get_setting('service_url', 'https://${service_host}/db')

    assert value == 'https://cool-db-server:6000/db'


def test_store_add_backend():
    store = Store([])

    store.add_backend(DictBackend(environment='staging'))

    assert store.get_setting('environment') == 'staging'



"""Retrieve settings and secrets from different storages."""

from .backends import AwsSsmBackend, DockerSecretBackend, DotenvBackend, EnvVarBackend
from .store import Store, SettingNotFoundException

__all__ = [
    'Store',
    'SettingNotFoundException',
    'EnvVarBackend',
    'DotenvBackend',
    'DockerSecretBackend',
    'AwsSsmBackend',
]

__version__ = '0.6.1'



import re


class SettingNotFoundException(Exception):
    """Error raised for settings not found in any backend and without default value."""


_no_default = object()


class Store(object):
    """A collection of backends that let you retrieve settings from them.

    Backends can be passed to constructor and/or added with add_backend.
    When get_setting is called, backends are searched in addition order
    to find the value; if no backend returns a value and the method was
    called without a default, SettingNotFoundException is raised.

    Interpolation is supported: include a setting value in another setting
    by surrounding the setting name with "${" and "}", example:

        # base setting
        redis_url = https://redis/
        stats_url = ${redis_url}1
        # => stats_url is https://redis/1
    """

    def __init__(self, backends):
        self._backends = tuple(backends)

    def add_backend(self, backend):
        self._backends += (backend,)

    def get_setting(self, key, default=_no_default):
        for backend in self._backends:
            ret = backend.get_setting(key)
            if ret is None:
                continue

            if "${" in ret:
                ret = self.interpolate(ret)

            return ret

        if default is not _no_default:
            if isinstance(default, str) and "${" in default:
                return self.interpolate(default)
            return default
        else:
            raise SettingNotFoundException(
                "Couldn't find setting {} in any backend".format(key)
            )

    def interpolate(self, string):
        res = re.sub(r"\$\{([_a-zA-Z0-9]+)\}", self._replace, string, count=1)
        return res

    def _replace(self, matchobj):
        key = matchobj.group(1)
        return self.get_setting(key)



from .awsssm import AwsSsmBackend
from .docker_secret import DockerSecretBackend
from .dotenv import DotenvBackend
from .env_var import EnvVarBackend

__all__ = [
    'EnvVarBackend',
    'DotenvBackend',
    'DockerSecretBackend',
    'AwsSsmBackend',
]



#!/usr/bin/env python
# setup.py generated by flit for tools that don't yet use PEP 517

from distutils.core import setup

packages = \
['configstore', 'configstore.backends']

package_data = \
{'': ['*']}

extras_require = \
{'awsssm': ['boto3'], 'dotenv': ['django-dotenv']}

setup(name='configstore',
      version='0.6.1',
      description='Retrieve settings and secrets from different storages.',
      author='Caravan Coop',
      author_email='hi@caravan.coop',
      url='https://github.com/caravancoop/configstore',
      packages=packages,
      package_data=package_data,
      extras_require=extras_require,
     )



import os
import errno

SECRETS_PATH = '/run/secrets'


class DockerSecretBackend(object):
    """Backend for docker secrets.

    See the documentation for docker services for more info.
    """

    def __init__(self, secrets_path=SECRETS_PATH):
        self.secrets_path = secrets_path

    def get_setting(self, key):
        path = os.path.join(self.secrets_path, key)

        try:
            file = open(path)
        except IOError as exc:
            if exc.errno == errno.ENOENT:
                return None
            else:
                raise

        with file:
            return file.readline().strip()



import pytest
import configstore

DOTENV_CONTENTS = u'''
SECRET_KEY=1234dot
ENVIRONMENT=PRODUCTION
'''

DOTENV_EMPTY = u'''

'''


def test_configstore_envvars_override_dotenv(monkeypatch, tmp_path):
    monkeypatch.setenv('ENVIRONMENT', 'STAGING')
    path = tmp_path / 'config.env'
    path.write_text(DOTENV_CONTENTS)

    store = configstore.Store([
        configstore.EnvVarBackend(),
        configstore.DotenvBackend(str(path)),
    ])

    secret_key = store.get_setting('SECRET_KEY')
    environment = store.get_setting('ENVIRONMENT')

    assert secret_key == '1234dot'
    assert environment == 'STAGING'


def test_configstore_dotenv_overrides_envvars(monkeypatch, tmp_path):
    monkeypatch.setenv('ENVIRONMENT', 'STAGING')
    path = tmp_path / 'config.env'
    path.write_text(DOTENV_CONTENTS)

    store = configstore.Store([
        configstore.DotenvBackend(str(path)),
        configstore.EnvVarBackend(),
    ])

    secret_key = store.get_setting('SECRET_KEY')
    environment = store.get_setting('ENVIRONMENT')

    assert secret_key == '1234dot'
    assert environment == 'PRODUCTION'


def test_dotenv_empty_file(monkeypatch, tmp_path):
    monkeypatch.setenv('ENVIRONMENT', 'STAGING')
    path = tmp_path / 'config.env'
    path.write_text(DOTENV_EMPTY)

    store = configstore.Store([
        configstore.DotenvBackend(str(path)),
        configstore.EnvVarBackend(),
    ])

    with pytest.raises(Exception) as excinfo:
        store.get_setting('DEBUG')
    environment = store.get_setting('ENVIRONMENT')

    assert "Couldn't find setting" in str(excinfo.value)
    assert environment == 'STAGING'


def test_interpolation(monkeypatch, tmp_path):
    monkeypatch.setenv('ENVIRONMENT', 'STAGING')
    monkeypatch.setenv('REDIS_URL', 'https://redis:4012')
    #monkeypatch.setenv('APP_NAME', 'supertest-${ENVIRONMENT}-1')
    path = tmp_path / 'config.env'
    path.write_text(DOTENV_CONTENTS)
    path.write_text('APP_NAME=supertest-${ENVIRONMENT}-1')

    store = configstore.Store([
        configstore.EnvVarBackend(),
        configstore.DotenvBackend(str(path)),
    ])

    environment = store.get_setting('ENVIRONMENT')
    app_name = store.get_setting('APP_NAME')
    session_url = store.get_setting('SESSION_URL', '${REDIS_URL}/2')

    assert environment == 'STAGING'
    assert app_name == 'supertest-STAGING-1'
    assert session_url == 'https://redis:4012/2'
